local OPCODE = 61
local window, creatureBarHP, creatureHP, creatureName = nil
local focusedbos = 0
local focusedMob = 0

function init()
	connect(g_game, {
		onGameStart = create,
		onGameEnd = destroy,
		onAttackingCreatureChange = onAttackingCreatureChange,
		onAppear = onCreatureAppear,
		onDisappear = onCreatureDisappear
	})
	connect(Creature, {
		onHealthPercentChange = onHealthPercentChange,
		onSpecialPercentChange = onSpecialPercentChange,
		onAppear = onCreatureAppear,
		onDisappear = onCreatureDisappear
	})

	if g_game.isOnline() then
		create()
	end
end

function terminate()
	disconnect(g_game, {
		onGameStart = create,
		onGameEnd = destroy,
		onAttackingCreatureChange = onAttackingCreatureChange,
		onAppear = onCreatureAppear,
		onDisappear = onCreatureDisappear
	})
	disconnect(Creature, {
		onHealthPercentChange = onHealthPercentChange,
		onSpecialPercentChange = onSpecialPercentChange,
		onAppear = onCreatureAppear,
		onDisappear = onCreatureDisappear
	})
	destroy()
end

function create()
	if window then
		return
	end

	window = g_ui.loadUI("bosbar", modules.game_interface.getMapPanel())

	window:hide()

	creatureBarHP = window:recursiveGetChildById("creatureBarHP")
	creatureName = window:recursiveGetChildById("creatureName")
	creatureHP = window:recursiveGetChildById("creatureHP")
	creatureSpecial = window:recursiveGetChildById("special")
end

function destroy()
	if window then
		window:destroy()

		window = nil
		creatureBarHP = nil
		creatureHP = nil
		creatureOutfit = nil
		creatureName = nil
		creatureSpecial = nil
		focusedbos = 0
		focusedMob = 0
	end
end

function onExtendedOpcode(protocol, code, buffer)
	if not g_game.isOnline() then
		return
	end

	local json_status, json_data = pcall(function ()
		return json.decode(buffer)
	end)

	if not json_status then
		g_logger.error("[bos Bar] JSON error: " .. data)

		return false
	end

	if json_data.action == "show" then
		show(json_data.data)
	elseif json_data.action == "hide" then
		hide()
	end
end

function show(data)
	focusedbos = data.cid
	creatureName:setText(data.name)
	creatureHP:setText(data.health .. "%")
	creatureSpecial:setPercent(data.health)
	window:show()
end

function hide()
	focusedbos = 0
	focusedMob = 0

	window:hide()
end

bosBarEnabled = true

function setEnabled(value)
	bosBarEnabled = value
end


local bosNames = {
    "Staff Twister",
    "Demodras",
	"Draconian",
	"Ferumbras",
	"Hellgorak",
	"Hornedfox",
	"King of Sea",
	"Malice",
	"Orshabaal",
	"Megalomania",
	"Ushuriel"
}

function checkbosName(name)
    for _, bosName in ipairs(bosNames) do
        if string.lower(name) == string.lower(bosName) then
            return true
        end
    end
    return false
end

function onAttackingCreatureChange(creature, oldCreature)
    if bosBarEnabled then
        if focusedbos ~= 0 then
            return
        end
        if creature and checkbosName(creature:getName()) then
            creatureHP:setText(creature:getHealthPercent() .. "%")
            creatureSpecial:setPercent(creature:getHealthPercent())
            focusedMob = creature:getId()
            window:show()
        else
            hide()
        end
    else
        hide()
    end
end


function onCreatureAppear(creature)
    if bosBarEnabled then
        if focusedbos ~= 0 then
            return
        end
        if creature and checkbosName(creature:getName()) then
            creatureName:setText(creature:getName())
            creatureHP:setText(creature:getHealthPercent() .. "%")
            creatureSpecial:setPercent(creature:getHealthPercent())
            focusedMob = creature:getId()
            window:show()
        end
    end
end

function onCreatureDisappear(creature)
    if bosBarEnabled and (creature:getId() == focusedMob or creature:getId() == focusedbos) then
        hide()
    end
end

function onHealthPercentChange(creature, health)
	if bosBarEnabled then
		if focusedbos == creature:getId() or focusedMob == creature:getId() then
			creatureHP:setText(health .. "%")
			creatureSpecial:setPercent(health)
		end
	else
		hide()
	end
end

function onSpecialPercentChange(creature, special)
	if special > 0 then
		if not creatureSpecial:isVisible() then
			creatureSpecial:setVisible(true)
		end
		creatureSpecial:setPercent(special)
	else
		creatureSpecial:setVisible(false)
	end
end

